@@ -50,13 +50,13 @@ impl<'a> Context<'a> {
                     Identifier::from_full_name("ans"),
                     Box::new(Expr::Unit(
                         num.unit.clone(),
-                        Box::new(Expr::Literal(num.to_f64())),
+                        Box::new(crate::ast::build_literal_ast(&num)),
                     )),
                 )
             } else {
                 Stmt::VarDecl(
                     Identifier::from_full_name("ans"),
-                    Box::new(Expr::Literal(num.to_f64())),
+                    Box::new(crate::ast::build_literal_ast(&num)),
                 )
             });
 
@@ -175,12 +175,9 @@ fn eval_unary_expr(
     let num = eval_expr(context, &expr, unit)?;
 
     match op {
-        TokenKind::Minus => Ok(KalkNum::new(-num.value, &num.unit)),
-        TokenKind::Percent => Ok(KalkNum::new(num.value * 0.01, unit)),
-        TokenKind::Exclamation => Ok(KalkNum::new(
-            prelude::special_funcs::factorial(num.value),
-            unit,
-        )),
+        TokenKind::Minus => Ok(num.mul(context, KalkNum::from(-1f64))),
+        TokenKind::Percent => Ok(num.mul(context, KalkNum::from(0.01f64))),
+        TokenKind::Exclamation => Ok(prelude::special_funcs::factorial(num)),
         _ => Err(CalcError::InvalidOperator),
     }
 }
@@ -212,9 +209,11 @@ pub fn convert_unit(
             Box::new(expr.clone()),
         ));
 
-        Ok(KalkNum::new(
-            eval_expr(context, &unit_def, "")?.value,
+        let num = eval_expr(context, &unit_def, "")?;
+        Ok(KalkNum::new_with_imaginary(
+            num.value,
             to_unit.into(),
+            num.imaginary_value,
         ))
     } else {
         Err(CalcError::InvalidUnit)
@@ -279,14 +278,14 @@ pub(crate) fn eval_fn_call_expr(
                 prelude::call_unary_func(
                     context,
                     &identifier.full_name,
-                    x.value,
+                    x,
                     &context.angle_unit.clone(),
                 )
             }
         }
         2 => {
-            let x = eval_expr(context, &expressions[0], "")?.value;
-            let y = eval_expr(context, &expressions[1], "")?.value;
+            let x = eval_expr(context, &expressions[0], "")?;
+            let y = eval_expr(context, &expressions[1], "")?;
             prelude::call_binary_func(
                 context,
                 &identifier.full_name,
@@ -298,11 +297,8 @@ pub(crate) fn eval_fn_call_expr(
         _ => None,
     };
 
-    if let Some((result, func_unit)) = prelude_func {
-        return Ok(KalkNum::new(
-            result,
-            if unit.len() > 0 { unit } else { &func_unit },
-        ));
+    if let Some((result, _)) = prelude_func {
+        return Ok(result);
     }
 
     // Special functions
@@ -323,7 +319,9 @@ pub(crate) fn eval_fn_call_expr(
 
             for n in start..=end {
                 context.sum_n_value = Some(n);
-                sum.value += eval_expr(context, &expressions[2], "")?.value;
+                let eval = eval_expr(context, &expressions[2], "")?;
+                sum.value += eval.value;
+                sum.imaginary_value += eval.imaginary_value;
             }
 
             context.sum_n_value = None;
@@ -331,17 +329,34 @@ pub(crate) fn eval_fn_call_expr(
 
             return Ok(sum);
         }
-        "integrate" | "∫" => {
-            // Make sure exactly 3 arguments were supplied.
-            if expressions.len() != 3 {
-                return Err(CalcError::IncorrectAmountOfArguments(
+        "integrate" | "integral" | "∫" => {
+            // Make sure either 3 or 4 arguments were supplied.
+            if expressions.len() < 3 || expressions.len() > 4 {}
+
+            return match expressions.len() {
+                3 => calculus::integrate_with_unknown_variable(
+                    context,
+                    &expressions[0],
+                    &expressions[1],
+                    &expressions[2],
+                ),
+                4 => calculus::integrate(
+                    context,
+                    &expressions[0],
+                    &expressions[1],
+                    &expressions[2],
+                    if let Expr::Var(integration_variable) = &expressions[3] {
+                        &integration_variable.full_name[1..]
+                    } else {
+                        return Err(CalcError::ExpectedDx);
+                    },
+                ),
+                _ => Err(CalcError::IncorrectAmountOfArguments(
                     3,
                     "integrate".into(),
                     expressions.len(),
-                ));
-            }
-
-            return calculus::integrate(context, &expressions[0], &expressions[1], &expressions[2]);
+                )),
+            };
         }
         _ => (),
     }
@@ -436,7 +451,6 @@ mod tests {
     }
 
     fn cmp(x: KalkNum, y: f64) -> bool {
-        println!("{} = {}", x.to_f64(), y);
         (x.to_f64() - y).abs() < 0.0001
     }
 
