@@ -29,8 +29,10 @@ pub struct Context {
     unit_decl_base_unit: Option<String>,
     parsing_identifier_stmt: bool,
     equation_variable: Option<String>,
-    contains_equal_sign: bool,
+    contains_equation_equal_sign: bool,
     is_in_integral: bool,
+    current_function: Option<String>,
+    current_function_parameters: Option<Vec<String>>,
 }
 
 #[wasm_bindgen]
@@ -47,8 +49,10 @@ impl Context {
             unit_decl_base_unit: None,
             parsing_identifier_stmt: false,
             equation_variable: None,
-            contains_equal_sign: false,
+            contains_equation_equal_sign: false,
             is_in_integral: false,
+            current_function: None,
+            current_function_parameters: None,
         };
 
         parse(&mut context, crate::prelude::INIT).unwrap();
@@ -112,7 +116,7 @@ pub enum CalcError {
 impl ToString for CalcError {
     fn to_string(&self) -> String {
         match self {
-            CalcError::ExpectedDx => format!("Expected eg. dx, to specify for which variable the operation is being done to. Example with integration: ∫(0, 1, x dx)."),
+            CalcError::ExpectedDx => format!("Expected eg. dx, to specify for which variable the operation is being done to. Example with integration: ∫(0, 1, x dx) or ∫(0, 1, x, dx). You may need to put parenthesis around the expression before dx/dy/du/etc."),
             CalcError::IncorrectAmountOfArguments(expected, func, got) => format!(
                 "Expected {} arguments for function {}, but got {}.",
                 expected, func, got
@@ -143,7 +147,9 @@ pub fn eval(
     input: &str,
     #[cfg(feature = "rug")] precision: u32,
 ) -> Result<Option<KalkNum>, CalcError> {
-    context.contains_equal_sign = input.contains("=");
+    // Variable and function declaration parsers will set this to false
+    // if the equal sign is for one of those instead.
+    context.contains_equation_equal_sign = input.contains("=");
     let statements = parse(context, input)?;
 
     let mut interpreter = interpreter::Context::new(
@@ -208,20 +214,28 @@ fn parse_identifier_stmt(context: &mut Context) -> Result<Stmt, CalcError> {
         if let Expr::FnCall(identifier, parameters) = primary {
             if !prelude::is_prelude_func(&identifier.full_name) {
                 advance(context);
-                let expr = parse_expr(context)?;
-                let mut parameter_identifiers = Vec::new();
 
                 // All the "arguments" are expected to be parsed as variables,
                 // since parameter definitions look the same as variable references.
                 // Extract these.
+                let mut parameter_identifiers = Vec::new();
                 for parameter in parameters {
                     if let Expr::Var(parameter_identifier) = parameter {
-                        parameter_identifiers.push(parameter_identifier.full_name);
+                        parameter_identifiers.push(format!(
+                            "{}-{}",
+                            &identifier.pure_name, &parameter_identifier.full_name
+                        ));
                     }
                 }
 
-                let fn_decl =
-                    Stmt::FnDecl(identifier.clone(), parameter_identifiers, Box::new(expr));
+                context.current_function = Some(identifier.pure_name.clone());
+                context.current_function_parameters = Some(parameter_identifiers.clone());
+                context.contains_equation_equal_sign = false;
+                context.equation_variable = None;
+                let expr = parse_expr(context)?;
+                let fn_decl = Stmt::FnDecl(identifier, parameter_identifiers, Box::new(expr));
+                context.current_function = None;
+                context.current_function_parameters = None;
 
                 // Insert the function declaration into the symbol table during parsing
                 // so that the parser can find out if particular functions exist.
@@ -241,6 +255,8 @@ fn parse_identifier_stmt(context: &mut Context) -> Result<Stmt, CalcError> {
 fn parse_var_decl_stmt(context: &mut Context) -> Result<Stmt, CalcError> {
     let identifier = advance(context).clone();
     advance(context); // Equal sign
+    context.contains_equation_equal_sign = false;
+    context.equation_variable = None;
     let expr = parse_expr(context)?;
     if inverter::contains_var(&context.symbol_table, &expr, &identifier.value) {
         return Err(CalcError::VariableReferencesItself);
@@ -448,7 +464,7 @@ fn parse_primary(context: &mut Context) -> Result<Expr, CalcError> {
         TokenKind::OpenParenthesis => parse_group(context)?,
         TokenKind::Pipe | TokenKind::OpenCeil | TokenKind::OpenFloor => parse_group_fn(context)?,
         TokenKind::Identifier => parse_identifier(context)?,
-        TokenKind::Literal => Expr::Literal(string_to_num(&advance(context).value)),
+        TokenKind::Literal => Expr::Literal(string_to_num(&advance(context).value)?),
         _ => return Err(CalcError::UnableToParseExpression),
     };
 
@@ -488,7 +504,7 @@ fn parse_identifier(context: &mut Context) -> Result<Expr, CalcError> {
     {
         // If there is a function with this name, parse it as a function, with the next token as the argument.
         if context.symbol_table.contains_fn(&identifier.pure_name) {
-            let parameter = parse_primary(context)?;
+            let parameter = parse_factor(context)?;
             return Ok(Expr::FnCall(identifier, vec![parameter]));
         }
     }
@@ -504,7 +520,9 @@ fn parse_identifier(context: &mut Context) -> Result<Expr, CalcError> {
     if !parse_as_var_instead && match_token(context, TokenKind::OpenParenthesis) {
         advance(context);
 
-        let is_integral = identifier.full_name == "integrate" || identifier.full_name == "∫";
+        let is_integral = identifier.full_name == "integrate"
+            || identifier.full_name == "integral"
+            || identifier.full_name == "∫";
         if is_integral {
             context.is_in_integral = true;
         }
@@ -526,11 +544,6 @@ fn parse_identifier(context: &mut Context) -> Result<Expr, CalcError> {
         return Ok(Expr::FnCall(identifier, parameters));
     }
 
-    // Eg. dx inside an integral, should be parsed as *one* identifier
-    if context.is_in_integral && identifier.full_name.starts_with("d") {
-        return Ok(Expr::Var(identifier));
-    }
-
     // Eg. x
     if parse_as_var_instead || context.symbol_table.contains_var(&identifier.pure_name) {
         Ok(Expr::Var(identifier))
@@ -540,44 +553,140 @@ fn parse_identifier(context: &mut Context) -> Result<Expr, CalcError> {
     } else {
         if let Some(equation_var) = &context.equation_variable {
             if &identifier.full_name == equation_var {
-                return Ok(Expr::Var(identifier));
+                return Ok(build_var(context, &identifier.full_name));
             }
-        } else if context.contains_equal_sign {
+        }
+        if context.contains_equation_equal_sign {
             context.equation_variable = Some(identifier.full_name.clone());
-            return Ok(Expr::Var(identifier));
+            return Ok(build_var(context, &identifier.full_name));
         }
 
-        let mut chars = identifier.pure_name.chars();
-        let mut left = Expr::Var(Identifier::from_full_name(
-            &chars.next().unwrap().to_string(),
-        ));
+        if identifier.pure_name.len() == 1 {
+            return Ok(build_var(context, &identifier.full_name));
+        }
 
-        // Turn each individual character into its own variable reference.
-        // This parses eg `xy` as `x*y` instead of *one* variable.
-        let mut right_chars = chars.peekable();
-        while let Some(c) = right_chars.next() {
-            // If last iteration
-            let right = if right_chars.peek().is_none() {
-                context.pos -= 1;
-                context.tokens[context.pos] = Token {
+        // Eg. dx inside an integral, should be parsed as *one* identifier
+        // Reverse the identifier and take two. This gets the last two characters (in reversed order).
+        // Now reverse this to finally get the last two characters in correct order.
+        // It's a bit weird, but it should work for more than ASCII.
+        let mut identifier_without_dx: Vec<char> = identifier.full_name.chars().collect();
+        let mut last_two_chars = String::new();
+        let last_char = identifier_without_dx.pop().unwrap_or_default();
+        let first_char = identifier_without_dx.pop().unwrap_or_default();
+        last_two_chars.push(first_char);
+        last_two_chars.push(last_char);
+
+        if context.is_in_integral && last_two_chars.starts_with("d") {
+            // If the token contains more than just "dx",
+            // save the dx/dy/du/etc. in a variable, that can be
+            // used further down when splitting the identifier into multiple variables.
+            if identifier.full_name.len() > 2 {
+                // This variable will be used further down in order to separate dx from the rest.
+                let pos = context.pos - 1;
+                context.pos = pos;
+                context.tokens[pos] = Token {
                     kind: TokenKind::Identifier,
-                    value: c.to_string(),
+                    value: identifier_without_dx.iter().collect(),
                     span: (0, 0),
                 };
 
-                parse_exponent(context)?
-            } else {
-                Expr::Var(Identifier::from_full_name(&c.to_string()))
-            };
+                let left_expr = parse_exponent(context)?;
 
-            left = Expr::Binary(Box::new(left), TokenKind::Star, Box::new(right));
+                // Revert back to how it was before.
+                context.tokens[pos] = Token {
+                    kind: TokenKind::Identifier,
+                    value: identifier.full_name.to_string(),
+                    span: (0, 0),
+                };
+
+                return Ok(Expr::Binary(
+                    Box::new(left_expr),
+                    TokenKind::Star,
+                    Box::new(Expr::Var(Identifier::from_full_name(&last_two_chars))),
+                ));
+            } else {
+                return Ok(Expr::Var(Identifier::from_full_name(&last_two_chars)));
+            }
         }
 
-        // TODO: When implementing derivation for variables, make sure to add the derivation here.
-        Ok(left)
+        split_into_variables(context, &identifier)
     }
 }
 
+fn split_into_variables(context: &mut Context, identifier: &Identifier) -> Result<Expr, CalcError> {
+    let mut chars: Vec<char> = identifier.pure_name.chars().collect();
+    let mut left = Expr::Var(Identifier::from_full_name(&chars[0].to_string()));
+
+    // Temporarily remove the last character and check if a function
+    // without that character exists. If it does,
+    // create a function call expression, where that last character
+    // is the argument.
+    let last_char = chars.pop().unwrap_or_default();
+    let identifier_without_last: String = chars.iter().collect();
+    if context.symbol_table.contains_fn(&identifier_without_last) {
+        return Ok(Expr::FnCall(
+            Identifier::from_full_name(&identifier_without_last),
+            vec![Expr::Var(Identifier::from_full_name(
+                &last_char.to_string(),
+            ))],
+        ));
+    } else {
+        // Otherwise, re-add the character.
+        chars.push(last_char);
+    }
+
+    // Turn each individual character into its own variable reference.
+    // This parses eg `xy` as `x*y` instead of *one* variable.
+    let mut right_chars = chars.iter().skip(1).peekable();
+    while let Some(c) = right_chars.next() {
+        // If last iteration
+        let right = if right_chars.peek().is_none() {
+            // Temporarily change the token content, so that
+            // the parse_exponent step will parse it as its
+            // new name. It will later be switched back,
+            // since the parser sometimes rewinds a bit,
+            // and may get confused by a sudden change.
+            let pos = context.pos - 1;
+            context.pos = pos;
+            context.tokens[pos] = Token {
+                kind: TokenKind::Identifier,
+                value: c.to_string(),
+                span: (0, 0),
+            };
+
+            let last_var = parse_exponent(context)?;
+
+            // Revert back to how it was before.
+            context.tokens[pos] = Token {
+                kind: TokenKind::Identifier,
+                value: identifier.full_name.to_string(),
+                span: (0, 0),
+            };
+
+            last_var
+        } else {
+            build_var(context, &c.to_string())
+        };
+
+        left = Expr::Binary(Box::new(left), TokenKind::Star, Box::new(right));
+    }
+
+    Ok(left)
+}
+
+fn build_var(context: &Context, name: &str) -> Expr {
+    if let (Some(function_name), Some(params)) = (
+        context.current_function.as_ref(),
+        context.current_function_parameters.as_ref(),
+    ) {
+        let identifier = Identifier::parameter_from_name(name, &function_name);
+        if params.contains(&identifier.full_name) {
+            return Expr::Var(identifier);
+        }
+    }
+    return Expr::Var(Identifier::from_full_name(name));
+}
+
 fn peek(context: &Context) -> &Token {
     &context.tokens[context.pos]
 }
@@ -615,8 +724,12 @@ fn is_at_end(context: &Context) -> bool {
     context.pos >= context.tokens.len() || peek(context).kind == TokenKind::EOF
 }
 
-fn string_to_num(value: &str) -> f64 {
-    value.replace(" ", "").parse::<f64>().unwrap()
+fn string_to_num(value: &str) -> Result<f64, CalcError> {
+    if let Ok(result) = value.replace(" ", "").parse::<f64>() {
+        Ok(result)
+    } else {
+        Err(CalcError::InvalidNumberLiteral(value.into()))
+    }
 }
 
 #[cfg(test)]
@@ -651,6 +764,37 @@ mod tests {
         assert_eq!(parse(tokens).unwrap(), Stmt::Expr(var("x")));
     }
 
+    #[test]
+    #[wasm_bindgen_test]
+    fn test_var_multiplication() {
+        let mut context = Context::new();
+        context.symbol_table.insert(Stmt::VarDecl(
+            Identifier::from_full_name("x"),
+            literal(1f64),
+        ));
+        context.symbol_table.insert(Stmt::VarDecl(
+            Identifier::from_full_name("y"),
+            literal(2f64),
+        ));
+
+        // xy²
+        let tokens = vec![
+            token(Identifier, "xy"),
+            token(Power, ""),
+            token(Literal, "2"),
+            token(EOF, ""),
+        ];
+
+        assert_eq!(
+            parse(tokens).unwrap(),
+            Stmt::Expr(binary(
+                var("x"),
+                Star,
+                binary(var("y"), Power, literal(2f64))
+            ))
+        );
+    }
+
     #[test]
     #[wasm_bindgen_test]
     fn test_binary() {
@@ -793,7 +937,7 @@ mod tests {
             token(Equals, ""),
             token(Literal, "1"),
             token(Plus, ""),
-            token(Literal, "2"),
+            token(Identifier, "x"),
             token(EOF, ""),
         ];
 
@@ -801,8 +945,8 @@ mod tests {
             parse(tokens).unwrap(),
             Stmt::FnDecl(
                 Identifier::from_full_name("f"),
-                vec![String::from("x")],
-                binary(literal(1f64), Plus, literal(2f64))
+                vec![String::from("f-x")],
+                binary(literal(1f64), Plus, param_var("f", "x"))
             )
         );
     }
